pickerlight = {
    'fg':'#1b1b1b','bg':'#fbfbfb',
    'outline':'#ececec','activefg':'#1b1b1b',
    'activebg':'#f6f6f6','onfg':'#eaecfb','onbg':'#0067C0',
    'buttonfg':'#1a1a1a','buttonbg':'#fbfbfb',
    'buttonactivefg':'#1a1a1a','buttonactivebg':'#f0f0f0',
    'buttononfg':'#1a1a1a','buttononbg':'#f3f3f3',
}

pickerdark = {
    'fg':'#cfcfcf','bg':'#2d2d2d','outline':'#3c3c3c',
    'activefg':'#ffffff','activebg':'#323232',
    'onfg':'#000000','onbg':'#4CC2FF',
    'buttonfg':'#ffffff','buttonbg':'#2d2d2d',
    'buttonactivefg':'#ffffff','buttonactivebg':'#383838',
    'buttononfg':'#ffffff','buttononbg':'#343434',
}